      ////////////////////////////////////////////////
     // THE CONSTRAINT-UNDERSTANDING SYSTEM         //
    //  (Version 2)                                ///
   // Written by Andrew Broad                     ////
  //                                             /////
 // DefinedTypeInstance.java (1999:11:25 12:08) //////
//////////////////////////////////////////////////////
// Methods of DefinedTypeInstance:              //////
// - DefinedTypeInstance constructor            //////
// - getDefinedType                             /////
// - getValue                                   ////
// - toString                                   ///
// - FILise                                     //
/////////////////////////////////////////////////

import java.io.*;
import java.util.*;

import jex.*;

/*****************************************************************************
** The class `DefinedTypeInstance' is used to wrap an instance of a defined
** type.
*****************************************************************************/

public class DefinedTypeInstance implements Serializable, FILisable {
	private DefinedType _definedType;
	private Object _value;

	/********************************************************************/
	// DefinedTypeInstance constructor : (DefinedType x Object) ->
	/*********************************************************************
	** Constructs this defined type instance by plugging in the given
	** value.
	** @param type The defined type of which this is an instance.
	** @param value The value of this defined type instance, which must
	**  conform to the underlying type.
	*********************************************************************/
 
 	public DefinedTypeInstance (DefinedType type, Object value) {
		_definedType= type;
		_value= value;
	}
	
	/********************************************************************/
	// DefinedTypeInstance constructor : (String x Object) ->
	/*********************************************************************
	** Constructs this defined type instance by plugging in the given
	** value. The defined type is specified by name for convenience.
	** @param definedTypeName The name of the defined type to be
	**                        instantiated.
	** @param value The value of this defined type instance, which must
	**  conform to the underlying type.
	*********************************************************************/
 
 	public DefinedTypeInstance (String definedTypeName, Object value) {
		_definedType= DefinedType.lookup(definedTypeName);
		_value= value;
	}
	
	/********************************************************************/
	// DefinedTypeInstance.getType : void -> DefinedType
	/*********************************************************************
	** Returns the defined type of which this object is an instance.
	** @return The defined type of which this is an instance.
	*********************************************************************/

	public DefinedType getDefinedType () {
		return _definedType;
	}

	/********************************************************************/
	// DefinedTypeInstance.getValue : void -> FILisable
	/*********************************************************************
	** Returns the value of this defined type instance.
	** @return The value of this defined type instance.
	*********************************************************************/

	public Object getValue () {
		return _value;
	}

	/********************************************************************/
	// DefinedTypeInstance.toString : void -> String
	/*********************************************************************
	** Returns a string representing this defined type instance.
	** @return The string representation of this defined type instance.
	*********************************************************************/

	public String toString () {
		return _value.toString();
	}

	/********************************************************************/
	// DefinedTypeInstance.FILise : boolean -> String
	/*********************************************************************
	** Returns a string representing this defined type instance in FIL.
	** @param HTML If true, the FIL will be augmented with HTML tags.
	** @return A string representing the defined type instance in FIL.
	*********************************************************************/

	public String FILise (boolean HTML) {
		try {
			Frame type= _definedType.getTypeFrame();
			//////////////////
			// Enumeration? //
			//////////////////
			if (type.getSlotFiller("tipe") instanceof
			   DefinedTypeInstance &&
			   (((DefinedTypeInstance)type.getSlotFiller("tipe")).
			     getDefinedType().getTypeFrame().
			     getSlotFiller("name").toString()).
			   equals("enumeration_type"))
			{
			   	return type.getSlotFiller("name")
			   	+ "(" + this + ")";
			}
			else {
				if (_value instanceof FILisable)
					return type.getSlotFiller("name")
					+ "("
					+ ((FILisable)_value).FILise(HTML)
					+ ")";
				else
					return type.getSlotFiller("name")
					+ "(" + _value + ")";
			}
		}
		catch (InvalidSlot is) {
			System.err.println("Something has gone badly " +
			"wrong for DefinedTypeInstance.FILise to throw an" +
			"InvalidSlot exception!");
			System.exit(1); return null;
		}
	}
}
