      /////////////////////////////////////////
     // THE CONSTRAINT-UNDERSTANDING SYSTEM  //
    //  (Version 2)                         ///
   // Written by Andrew Broad              ////
  //                                      ////
 // TabulateHTML.java (2002:11:20 15:43) ////
////////////////////////////////////////////
// Methods of TabulateHTML:             ///
// - main                               //
/////////////////////////////////////////

import java.io.*;
import java.util.*;

/*****************************************************************************
** The class `TabulateHTML' extracts experiment data from the CCUS output
** files out[1-8].txt, and synthesises an HTML table.
*****************************************************************************/

public class TabulateHTML implements Serializable {
	/********************************************************************/
	// static TabulateHTML.main : void -> void
	/*********************************************************************
	** Synthesise an HTML table from CCUS outputs.
	*********************************************************************/

	public static void main (String argv[]) {
		int[][] table= new int[8][11];

		for (int e= 0; e < 8; e++) {
			table[e]= tabulate("out" + (e+1) + ".txt");
		}

		System.out.println("<TABLE BORDER>");
		System.out.println("<TR><TD><B>Mode</B></TD><TD><B>E</B></TD>"
		+ "<TD><B>-T</B></TD><TD><B>-U</B></TD><TD><B>-S</B></TD><TD>"
		+ "<B>-H</B></TD><TD><B>-R</B></TD><TD><B>M</B></TD><TD><B>X"
		+ "</B></TD></TR>");
		System.out.print("<TR><TD><B>HLCs extracted from first model"
		+ "</B></TD>");
		for (int i= 0; i < 8; i++) {
			System.out.print("<TD><B>" + table[i][2]
			+ "</B></TD>");
		}
		System.out.println("</TR>");
		System.out.print("<TR><TD><B>HLCs extracted from second model"
		+ "</B></TD>");
		for (int i= 0; i < 8; i++) {
			System.out.print("<TD><B>" + table[i][3]
			+ "</B></TD>");
		}
		System.out.println("</TR>");
		System.out.print("<TR><TD><B>Comparisons"
		+ "</B></TD>");
		for (int i= 0; i < 8; i++) {
			System.out.print("<TD><B>" + table[i][4]
			+ "</B></TD>");
		}
		System.out.println("</TR>");
		System.out.print("<TR><TD><B>HLC-comparisons"
		+ "</B></TD>");
		for (int i= 0; i < 8; i++) {
			System.out.print("<TD><B>" + table[i][5]
			+ "</B></TD>");
		}
		System.out.println("</TR>");
		System.out.print("<TR><TD><B>Correspondences"
		+ "</B></TD>");
		for (int i= 0; i < 8; i++) {
			System.out.print("<TD><B>" + table[i][6]
			+ "</B></TD>");
		}
		System.out.println("</TR>");
		System.out.print("<TR><TD><B>HLC-correspondences"
		+ "</B></TD>");
		for (int i= 0; i < 8; i++) {
			System.out.print("<TD><B>" + table[i][7]
			+ "</B></TD>");
		}
		System.out.println("</TR>");
		System.out.print("<TR><TD><B>Differences"
		+ "</B></TD>");
		for (int i= 0; i < 8; i++) {
			System.out.print("<TD><B>" + table[i][8]
			+ "</B></TD>");
		}
		System.out.println("</TR>");
		System.out.print("<TR><TD><B>Number of FC-thesaurus lookups"
		+ "</B></TD>");
		for (int i= 0; i < 8; i++) {
			System.out.print("<TD><B>" + table[i][9]
			+ "</B></TD>");
		}
		System.out.println("</TR>");
		System.out.print("<TR><TD><B>Number of name-thesaurus lookups"
		+ "</B></TD>");
		for (int i= 0; i < 8; i++) {
			System.out.print("<TD><B>" + table[i][10]
			+ "</B></TD>");
		}
		System.out.println("</TR>");
		System.out.println("</TABLE>");
	}

	/********************************************************************/
	// static TabulateHTML.tabulate : void -> void
	/*********************************************************************
	** Extract experiment data from a CCUS output file.
	** @param filename The filename of the CCUS output file.
	** @return An array of data extracted from said file.
	*********************************************************************/

	public static int[] tabulate (String filename) {
		int[] expt= new int[11]; int e= 0;

		BufferedReader in= null; FileInputStream fis= null;
		try {
			in= new BufferedReader(new InputStreamReader
			         (fis= new FileInputStream(filename)));
		} catch (FileNotFoundException fnfe) {
			System.err.println("TabulateHTML.tabulate: FileNotFou"
			+ "ndException for \"" + filename + "\"");
			System.exit(1);
		}
		String line;
		int i,n;
		char c;
		try {
			while ((line= in.readLine()) != null) {
				try {
					if (line.substring(0,6).
					    equals("Number"))
					{
						i= 6; // start looking for ":"
						while (line.charAt(i) != ':'){
							i++;
						}
						i+= 2; // skip ": "
						n= Integer.parseInt
						   (line.substring(i,
						         line.length()));
						expt[e++]= n;
					}
				} catch (IndexOutOfBoundsException ioobe) {}
			}
			fis.close();
		} catch (IOException ioe) {
			System.err.println("TabulateHTML.tabulate: IOExceptio"
			+ "n thrown by BufferedReader.readLine()");
			System.exit(1);
		}
		return expt;
	}
}
