      ////////////////////////////////////////////
     // THE CONSTRAINT-UNDERSTANDING SYSTEM     //
    //  (Version 2)                            ///
   // Written by Andrew Broad                 ////
  //                                         /////
 // UnorderedVector.java (2002:01:14 19:04) //////
//////////////////////////////////////////////////
// Methods of UnorderedVector:              /////
// - UnorderedVector constructor()          ////
// - FILise                                 ///
// - subsetOf                               //
/////////////////////////////////////////////

import java.io.*;
import java.util.*;

import jex.*;

/*****************************************************************************
** The class `UnorderedVector' is a subclass of vectors that represent sets or
** bags in the frame system.
*****************************************************************************/

public class UnorderedVector    extends Vector
                             implements FILisable
{
	/********************************************************************/
	// UnorderedVector constructor : void ->
	/*********************************************************************
	** Constructs an empty unordered vector.
	*********************************************************************/
 
 	public UnorderedVector () {
		super();
	}
	
	/********************************************************************/
	// UnorderedVector.FILise : boolean -> String
	/*********************************************************************
	** Writes FIL for this unordered vector. Elements are enclosed in
	** curly brackets and separated by commas. Elements are FIL-ised
	** recursively.
	** @param HTML If true, FIL will be augmented with HTML tags.
	** @return The FIL representation of this unordered vector.
	*********************************************************************/

	public String FILise (boolean HTML) {
		String agg= "{";

		for (int i= 0; i < size(); i++) {
			Object element= elementAt(i);
			if (element instanceof FILisable)
				agg+= ((FILisable)element).FILise(HTML);
			else
				agg+= element;
			if (i < (size() - 1))
			agg+= ", ";
		}
		return agg + "}";
	}

	/********************************************************************/
	// UnorderedVector.subsetOf : UnorderedVector -> boolean
	/*********************************************************************
	** Returns true iff this vector is a subset of a given vector, i.e. if
	** every element in this vector matches some element in that vector.
	** @param that The vector it's testing whether this is a subset of.
	** @param entityPair The current entity-pair for the summarised output
	**                   (optional).
	** @return true if this vector is a subset of that vector, false if it
	**         is not.
	*********************************************************************/

	public boolean subsetOf (UnorderedVector that, EntityPair entityPair){
		ListIterator thisLI= listIterator();
		while (thisLI.hasNext()) {
			Object element1= thisLI.next();
			boolean match= false;
			ListIterator thatLI= that.listIterator();
			while (thatLI.hasNext()) {
				Object element2= thatLI.next();
				if (CCUS_FMA.matchFillers(element1, element2,
				                          entityPair))
				{
					match= true;
				}
			}
			if (! match) {
				return false;
			}
		}
		return true;
	}
}
